@@ -1,12 +1,11 @@
 package IRC::Server::Tree;
-our $VERSION = '0.05';
-
+$IRC::Server::Tree::VERSION = '0.061001';
 ## Array-type object representing a network map.
 
 use strictures 1;
 use Carp;
 
-use Scalar::Util 'blessed';
+use Scalar::Util 'blessed', 'reftype';
 use Storable     'dclone' ;
 
 sub new {
@@ -32,7 +31,7 @@ sub new {
       last BUILD
     }
 
-    if (ref $opt eq 'ARRAY') {
+    if (reftype $opt eq 'ARRAY') {
       ## Got a Tree as a raw ARRAY.
       ## No clone; keep refs to allow darker forms of magic
       $self = $opt;
@@ -48,7 +47,7 @@ sub new {
 sub add_node_to_parent_ref {
   my ($self, $parent_ref, $name, $arrayref) = @_;
 
-  $arrayref = [@$arrayref] if blessed $arrayref;
+  $arrayref = [ @$arrayref ] if blessed $arrayref;
 
   push @$parent_ref, $name, ($arrayref||=[]);
 
@@ -185,17 +184,17 @@ sub names_beneath {
   my @list = @$ref;
   my @names;
 
-  ## Recurse and accumulate names.
+  ## Iter and accumulate names.
   while (my ($node_name, $node_ref) = splice @list, 0, 2) {
     push(@names, $node_name);
     push(@names, @{ $self->names_beneath($node_ref) || [] });
   }
 
-  \@names
+  [ @names ]
 }
 
 sub trace {
-  my ($self, $server_name, $parent_ref) = @_;
+  my ($self, $server_name, $parent_ref, $dfs) = @_;
 
   ## A list of named hops to the target.
   ## The last hop is the target's name.
@@ -203,7 +202,21 @@ sub trace {
   $parent_ref = $self unless defined $parent_ref;
 
   my $index_route =
-    $self->trace_indexes($server_name, $parent_ref)
+    $dfs ?
+      $self->trace_indexes_dfs($server_name, $parent_ref)
+      : $self->trace_indexes($server_name, $parent_ref) ;
+
+  return unless $index_route;
+
+  $self->path_by_indexes($index_route, $parent_ref)
+}
+
+sub trace_dfs {
+  my ($self, $server_name, $parent_ref) = @_;
+
+  $parent_ref = $self unless defined $parent_ref;
+
+  my $index_route = $self->trace_indexes_dfs($server_name, $parent_ref)
     or return;
 
   $self->path_by_indexes($index_route, $parent_ref)
@@ -223,7 +236,7 @@ sub path_by_indexes {
     $cur_ref = $cur_ref->[$idx];
   }
 
-  \@names
+  [ @names ]
 }
 
 sub trace_indexes {
@@ -233,16 +246,18 @@ sub trace_indexes {
   ##
   ## We explore each path in the current node, and as we find new paths,
   ## we queue them to be explored after the current iteration.
-  ## (This is in contrast to depth-first techniques, where you recursively
+  ##
+  ## This is in contrast to depth-first techniques, where you recursively
   ## explore each deeper reference as you hit it, with the path-so-far
-  ## included in the call, until you have the path desired.)
+  ## included in the call, until you have the path desired.
+  ## See trace_indexes_dfs() for a DFS alternative.
   ##
   ## This is useful for cases like an IRC server tree, where there is
   ## an essentially arbitrary structure to the tree; any node may have
   ## any arbitrary number of child nodes (ad infinitum) and we have no
-  ## actual hints as to the possible path.
-  ##
-  ## (Hmm. Considering running networked maze-solver races...)
+  ## actual hints as to the possible path. Therefore we cannot predict
+  ## the most performant path-finding mechanism -- but we can optimize
+  ## towards finding paths closest to us more quickly.
   ##
   ## Defaults to operating on $self
   ## Return indexes into arrays describing the path
@@ -256,12 +271,13 @@ sub trace_indexes {
   ## Subtracting one from an index will get you the NAME value.
 
   ## A start-point.
-  my @queue = ( PARENT => ($parent_ref || $self) );
+  my @queue = ( '' => ($parent_ref || $self) );
 
   ## Our seen routes.
   my %route;
 
   my $parent_idx = 0;
+
   PARENT: while (my ($parent_name, $parent_ref) = splice @queue, 0, 2) {
 
     return [ $parent_idx+1 ] if $parent_name eq $server_name;
@@ -289,6 +305,39 @@ sub trace_indexes {
   return
 }
 
+=pod
+
+=for Pod::Coverage trace_indexes_dfs
+
+=cut
+
+sub trace_indexes_dfs {
+  my ($self, $server_name, $parent_ref, $route) = @_;
+
+  ## Depth first node finder.
+  ##  For every child node found, call self with current route.
+
+  $route      = [] unless $route;
+  $parent_ref = $self unless $parent_ref;
+
+  my @list = @$parent_ref;
+
+  my $cur_idx = 1;
+  while (my ($name, $subnode) = splice @list, 0, 2) {
+    return [ @$route, $cur_idx ] if $name eq $server_name;
+
+    my $next = $self->trace_indexes_dfs(
+      $server_name, $subnode,
+      [ @$route, $cur_idx ]
+    );
+    return $next if $next;
+
+    $cur_idx += 2;
+  }
+
+  return
+}
+
 sub print_map {
   my ($self, $parent_ref) = @_;
 
@@ -357,18 +406,16 @@ See the DESCRIPTION for a complete method list.
 
 =head1 DESCRIPTION
 
-This piece was split out of a pending project because it may prove 
-otherwise useful. See L<IRC::Server::Tree::Network> for higher-level 
-(and simpler) methods pertaining to manipulation of an IRC network 
-specifically; a Network instance also provides an optional 
-memory-for-speed tradeoff via memoization of traced paths.
-
-IRC servers are linked to form a network.
 An IRC network is defined as a 'spanning tree' per RFC1459; this module 
 is an array-type object representing such a tree, with convenient path 
 resolution methods for determining route "hops" and extending or shrinking 
 the tree.
 
+See L<IRC::Server::Tree::Network> for higher-level 
+(and simpler) methods pertaining to manipulation of an IRC network 
+specifically; a Network instance also provides an optional 
+memory-for-speed tradeoff via memoization of traced paths.
+
 An IRC network tree is essentially unordered; any node can have any 
 number of child nodes, with the only rules being that:
 
@@ -411,8 +458,8 @@ array-of-arrays:
   ],
 
 The methods provided below can be used to manipulate the tree and 
-determine hops in a path to an arbitrary node using a breadth-first 
-search.
+determine hops in a path to an arbitrary node using either breadth-first 
+or depth-first search.
 
 Currently routes are not memoized; that's left to a higher layer or 
 subclass.
@@ -500,7 +547,7 @@ Also see L</child_node_for>
 
 Returns the tree in list format.
 
-Not useful for most purposes and may be removed.
+(L</as_hash> is likely to be more useful.)
 
 =head2 child_node_for
 
@@ -525,7 +572,7 @@ Returns the deleted node.
   my $names = $tree->names_beneath( $parent_name );
   my $names = $tree->names_beneath( $parent_ref );
 
-Return an arrayref of all names in the tree beneath the specified parent 
+Return an ARRAY of all names in the tree beneath the specified parent 
 node.
 
 Takes either the name of a node in the tree or a reference to a node.
@@ -551,8 +598,9 @@ Prints a visualization of the network map to STDOUT.
 
   my $names = $tree->trace( $parent_name );
   my $names = $tree->trace( $parent_name, $start_ref );
+  my $names = $tree->trace( $parent_name, $start_ref, 'dfs' );
 
-Returns an arrayref of the names of every hop in the path to the 
+Returns an ARRAY of the names of every hop in the path to the 
 specified parent name.
 
 Starts tracing from the root of the tree unless a parent node reference 
@@ -560,14 +608,25 @@ is also specified.
 
 The last hop returned is the target's name.
 
+Specifying a true value as a third argument is the same as calling 
+L</trace_dfs>. Defaults to breadth-first as described in 
+L</trace_indexes>.
+
+=head2 trace_dfs
+
+A convenience method for using depth-first tracing. This is likely to be less
+efficient than the default breadth-first approach for most network layouts.
+
+This is the same as specifying a true third argument to L</trace>.
+
 =head2 trace_indexes
 
-Primarily intended for internal use. This is the breadth-first search 
-that other methods use to find a node. There is nothing very useful you 
-can do with this externally except count hops; it is documented here to 
-show how path resolution works.
+Primarily intended for internal use. This is the BFS/DFS search 
+that other methods use to find a node. There is nothing very 
+useful you can do with this externally except count hops; it is documented 
+here to show how path resolution works.
 
-Returns an arrayref consisting of the index of every hop taken to get to 
+Returns an ARRAY consisting of the index of every hop taken to get to 
 the node reference belonging to the specified node name starting from 
 the root of the tree or the specified parent node reference.
 
